import csv
import gzip

infile_name = "../feature/GCF_002880755.1_Clint_PTRv2_feature_table.txt.gz"
outfile_name = "../gene/pan_troglodytes.tsv"
ens_fields = ("Protein ID", "Gene ID", "Region Type", "Region", 
              "Start", "End", "Strand", "Label", "Description")

def main():
    with gzip.open(infile_name, "rt", newline="") as infile, \
         open(outfile_name, "w", newline="") as outfile:
        assert infile.read(2) == "# "
        reader = csv.DictReader(infile, delimiter="\t")
        writer = csv.DictWriter(outfile, delimiter="\t", fieldnames=ens_fields)
        outfile.write("\t".join(ens_fields) + "\n")
        genes = {}
        for feature in reader:
            if feature["assembly_unit"] != "Primary Assembly":
                # if "GCF_000001635" not in str(infile_name):  # weird mouse :(
                continue
            if feature["feature"] == "gene" and feature["class"] == "protein_coding":
                genes[feature["GeneID"]] = feature
            if feature["feature"] == "CDS" and feature["class"] == "with_protein":
                writer.writerow(gbk2ens(feature, genes[feature["GeneID"]]))

def gbk2ens(feature, gene):
    region = (
        feature["chromosome"]
        if feature["seq_type"] == "chromosome"
        else feature["genomic_accession"]
    )
    return {
        "Protein ID":    feature["product_accession"], # .split(".")[0],
        # "Transcript ID": feature["related_accession"].split(".")[0],
        "Gene ID":       feature["GeneID"],
        "Region Type":   feature["seq_type"],
        "Region":        region,
        "Start":         gene["start"],
        "End":           gene["end"],
        "Strand":        {"+": "1", "-": "-1"}[feature["strand"]],
        "Label":         feature["symbol"],
        "Description":   gene["name"] or feature['name'],
    }

if __name__ == "__main__":
    main()

