// Solution: forblast   Project: posthit
#ifndef POSTHIT_H
#define POSTHIT_H
#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_WARNINGS

#ifdef _DEBUG
#define NOMPI
#endif

//#define NOMPI           // uncomment to obtain version w/o mpi
//#define __unix__

#ifdef __unix__
#define LINUX
#elif defined(WIN32) || defined(_WIN32) || defined(_WIN64)
#define WINDOWS
#endif

#ifndef NOMPI
#include <mpi.h>
#endif
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <cerrno>
#include <time.h>
#include <memory.h>
#include <assert.h>
#ifdef WINDOWS
#include <process.h>
#include <direct.h>
#include <io.h>			// for _access
#else	// linux
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif	// Windows/linux 

using namespace std;

#ifdef NOMPI
#define getTimer()  (double)clock()/CLOCKS_PER_SEC
#else
#define getTimer()  (mpisize>1 ? MPI_Wtime() : (double)clock()/CLOCKS_PER_SEC)
#endif

#ifdef WINDOWS
const char slash = '\\';
const char slash2 = '/';
#define isFile(a)	_access((a), 0)		// only check existense
#define PERMISSIONS			// no special permission in Windows
#else
const char slash = '/';
const char slash2 = '/';
#define isFile(a)	access((a), F_OK)
#define PERMISSIONS	umask(002);		// -> rw-rw-r--
#endif	// Windows/linux 
const string str_slash(1, slash);

typedef	map <string, int>	Dict;		// protein and gene dictionary type
typedef pair <string, int>	DictVal;	// dictionary entry
typedef Dict::iterator		DictIt;		// dictionary iterator type
typedef map <int, int>		Index;		// gene matrix index, gene_no->row_no|column_no
typedef pair <int, int>		IndexVal;	// index entry
typedef Index::iterator		IndexIt;	// index iterator type

const string    version = "1.8";
const string    comment = ";#";
const string    blank = " \t\n\r";
const string    delim = " |\t\n\r";
const string	eol = "\n\r";
const char      tab = '\t';
const char		star = '*';
const int       buflen = 16384;
const int       maxw = 36;              // max width of species name
const int       norank = 999;
const double	nonzero = 0.001;

const string    def_inpath = "hits";	// input path
const string	def_inmid = "-";		// interspecies connector
const string	def_inext = ".tsv";		// input extension
const string	def_dupath = "dup";		// file path of duplicates
const string	def_dupext = ".tsv";	// extension of duplicates
const string	def_genepath = "gene";	// path of gene tables
const string	def_geneext = ".tsv";	// extension of gene tables
const string	def_outpath = "homolog";	// output path
const string	def_outext = ".tsv";	// output extension
const int		def_usedup = 1;			// expand with duplicates: 1=yes, 0=no
const int		def_aggregate = 1;		// aggregate proteins->genes: 1=yes, 0=no
const int		def_symmetry = 0;		// symmetrize the matrices: 1=yes, 0=no
const int		def_useweight = 0;		// calculate and use weight: 1=yes, 0=no
const int		def_verbose = 0;		// verbose output to console: 1=yes, 0=no
const int		def_maxhomo = 0;		// max number of best homologs, 0=unlimited
const int		def_addrank = 0;		// add rank fields: 0=no, 1=yes, 2=both ranks
const float     def_ratio = 0.0;        // 1-score/max threshold, 0=usual rank
const string	def_pfield = "Protein ID";	// name of protein column in gene tables
const string	def_gfield = "Gene ID";		// name of gene column in gene tables
const string	def_rfield = "Region";		// name of region column in gene tables
const string	def_sfield = "Start";		// name of starts column in gene tables
const string	def_hsortmode = "s1Wi";	// horizontal sort mode: 1(self) n i p s w e
const string	hmodes = "1ipsweIPSWE";	// (none) self id position spp weight/score E-value
const string	def_vsortmode = "si";	// vertical sort mode: n i p s 
const string	vmodes = "ipsIPS";		// none id position spp 
const string    unspec = "_unspecified_";

struct Protein		// protein entry
{
	string		id;			// protein id
	int			sp;			// species no.
	int			gno;		// gene no.
	int			master;		// master copy no. of this protein (-1 means itself)
	int			rno;		// row no. in protein matrix (before sort)
	float		self;		// self-score
	vector <int> dup;		// duplicate protein nos. of this master copy
	Protein()	{ master = -1; rno = -1; self = 0.0; }
};

struct Gene			// gene entry
{
	string		id;
	string		region;
	int			start;
	int			sp;			// species no.
	vector <int> pno;		// protein nos.
};

struct Species		// list of species entry
{
	string		name;		// short alias of the species
	//string		fasta;		// fasta file name - not needed
};

struct Cell			// cell of the protein/gene matrix
{
	int		jno;			// 2nd protein/gene no.
	float	score;			// score or weight
	double	eval;			// E-value
public:
	Cell()
	{
		jno = -1;
		score = -1e5;
		eval = 1e5;
	}
	Cell(const Cell & mycell)
	{
		jno = mycell.jno;
		score = mycell.score;
		eval = mycell.eval;
	}
};

struct Row			// row of the protein/gene matrix
{
	int		ino;			// 1st protein/gene no.
	vector <Cell> cell;		// vector of cells in this row
};

struct Brank        // cell of the backward rank matrix
{
    int     ino;            // 1st protein/gene no.
    float   rank;           // rank of jno->ino homology (or norank|1.0)
};

struct Column       // column of the rank matrix
{
    int     jno;            // 2nd protein/gene no.
    vector <Brank> brank;   // vector of backward ranks in this column
};

enum ReturnCode
{
	RETCODE_ERROR = -2, RETCODE_NONE, RETCODE_OK,
	RETCODE_EARG, RETCODE_ELIST, RETCODE_EINPUT, RETCODE_EOTHER,
	RETCODE_EMATRIX, RETCODE_ERESULT
};

void    showHelp (const string& program);
void	argError (char arg);
void	argError (const string & msg);
void	refineArguments (void);
bool	ensureDirectory (const string& dir);
bool    readSpeciesList (const string& file, vector <Species>& species);
bool	readGeneTable (const int sp);
bool    understandHits(void);
bool	readResult (const int sp);
int		findField (const string& str, const string& field);
bool	fillSelfScore (const int sp);
int		countSelf (void);
void	fillProtMatrix (const int sp);
void    fillProtMatrix2 (const int sp);
void	expandProtMatrix (const int sp);
void	aggregateGeneMatrix (const int sp);
void	outputResult (const int sp);
bool    writeBackRank (const int sp);
bool	sortCells (const Cell& c1, const Cell& c2);
bool	sortRows (const Row& r1, const Row& r2);

#endif	// ifndef POSTHIT_H